function simseries = N_model(modeltype,ss_mode,numdays,minN,maxN,incr,mr,kd,initialR,initialOM,initialPMass)

%% File is the model in the manuscript "Plasticity in resource allocation and nitrogen use
%% efficiency in riparian vegetation:  implications for nitrogen retention," by  J. D. Schade &
%% D. B. Lewis.  Initial conceptual outline of model and all
%% empirical inputs come from Schade; conceptual improvements derive from
%% teamwork by Schade and Lewis; and model code and documentation are by Lewis.

%% This program considers an ecosystem comprised of a single baccharis plant growing
%% on a southwestern stream gravel bar. We model the proportion of N input that is
%% retained, and we discriminate between the proportion retained via plant uptake and the
%% proportion retained via denitrification. Additionally, we evaluate how these measures
%% of N retention compare between a model in which plant physiology is insensitive to N
%% concentration in the environment (the CONSTANT model) vs. a model in which plant physiology
%% changes with N concentration in the environment (the RESPONSE) model.  In the response model,
%% N concentration in the environment determines the proportion of whole plant production that
%% is allocated to roots and determines the N content within plant tissue (by contrast,
%% allocation of productivity to roots and plant tissue N content are set to constant values
%% in the constant model).

%% We develop a mass balance for "free N," which is N available to plants and
%% denitrifiers.  Conceptually, this N includes NO3, NH4, and simple organic N for
%% the plants and NO3 for the denitrifiers.  The mass balance equation is:

%% dNfr/dt = Nhyd + Nrec - Ndnt - Nupt - Nexp

%% where Nfr is the pool of dissolved "free" N (g), and all other terms are RATES (g/day)
%% of N input or output.  Note, these rate terms are not instantaneous rates, but rather
%% daily rates - e.g., Nrec is the mass of N recycled on any given day of the model simulation.
%% Dividing both sides of the above equation by time (i.e., by "day"), therefore, these rate
%% values could also be thought of as pools.  On any day, j, Nfr (a mass) is the balance
%% of several other masses (Nhyd, Nrec, Ndnt, etc).  Nfr is set to always equal zero, and
%% the values of the other masses (e.g., the amount (g) of recycled N, the amount of denitrified
%% N, etc.) change each day.

%% Definitions of parameters:  Nhyd is hydrologic input of N, Nrec is recycled N input from
%% decomposition of organic matter (where we assume all organic matter is dead roots), Ndnt
%% is N lost via denitrification, Nupt is N lost via plant uptake, and Nexp is N exported
%% via leaching.  We calculate Nexp by difference, forcing dNfr/dt=0.
%% By forcing dNfr/dt=0, we assume that there is no build up of available ("free") N
%% in the ecosystem (i.e., no mineral N accumulates in the vadose or transient storage zones
%% of the gravel bar because it all gets flushed by hydrologic discharge).

% ***** set model type: 0 for constant model; 1 for response model *****
% modeltype=0; % user defined in "function file" mode; uncomment this line when running script mode
% **********************************************************************

% *** determine discharge through "ecosystem" (i.e., discharge across the
% *** cross-section to be filled by grown plant)
flowvel = 1; % velocity of flow through gravel bar (m/h)
plant_xsec = 0.158; % eventual cross-sectional area (m^2) of zone influenced by BIG plant
% To evaluate N retention, we determine N inputs - N outputs across a plane. Though the model
% simulates the growth of a plant (and associated development of rooting zone with organic
% matter and denitrifiers), we hold constant the size of this plane across which retention
% is calculated.  Thus, the plant_xsec plane is fixed to an area equal to 'the area to be spanned'
% by root + organic matter of a the fully grown plant.
Q = flowvel * plant_xsec * 24; % hydro discharge through gravel bar (m^3/d)
% **********************************************************************************

% ******* parameters for P - whole plant productivity rate *********************
% ** (actually determine P below, after removing N loss from denitrification) **
Pmax = 10; %max productvity (g/day)
kp = 0.42; %half-sat const in relationship of productivity vs. plant-available N
% ******************************************************************************

% ** parameters for determining Ncont_new - proportional N content of newly produced plant tissue **
Ncontmax = 0.04; %max value for proportional N content of newly produced plant tissue
kNcont = 0.25; %half-sat const in relationship of plant new tissue N vs. plant-available N
% **********************************************************************************

% ** parameters for determining propR, which is the proportion
% ** of newly produced plant mass allocated to roots
c = 0.42; %intercept parameter in relationship of propR vs. plant-available N
k1 = -1.74; %decay parameter in relationship of propR vs plant-available
% **********************************************************************************

% ** parameters for turnover of root and OM pools **********************************
% these parameters are user-defined in the script file 'run_Nmodel' that calls this
% file, they are
% mr = rate constant (1/day) for root death (proportion of root mass dying each day), and
% kd = 0.00252; % rate constant (1/day) in the differential equation that
% describes the decomposition of organic matter (OM).
% This equation is the negative exponential model dOM/dt = c*exp(-kd*t).
% A rate of kd = 0.00252 per day yields a curve wherein 50% of OM is left after 275 days
% and 1% is left after 1825 days (5 years), assuming no new OM inputs.
% **********************************************************************************

Nc_list = (minN:incr:maxN); % list of N concentration values at which model is to be run, where
% N concentration refers to the concentration of N in stream water entering the gravel bar
Nlevels=length(Nc_list); % provides the number of N concentration values in Nc_list

for n=1:Nlevels; % use n=1:200 for linear plot, and n=1:201 for nonlinear plot
    % This "for n=..." loop is used to run the model at several values of the stream water
    % concentration of available N , [Nc].  In this "for" loop, each value of n (little n)
    % establishes a new [Nc].  Nested within each round of this "for" loop (i.e., within each
    % value of [Nc]) is a different "for" loop, namely "for day = ..."  This "for day=..." loop
    % specifies the number days to run the simulation at each [Nc] value.
    
    % For the given [Nc] value, first specify the things that will be constant for all
    % days ([Nc] and Nhyd)
    
    Nc = Nc_list(n); % nitrogen concentration (g/m3 = mg/L) input to gravel bar
    %for linear plot use Nc=0.005*n; for nonlin plot use pow = -(201-n)/100; Nc = 10^pow;
    Nhyd = Q * Nc; % hydrologic input (g/day) of N to gravel bar
    
    % Also specify starting values for variables that will change through time (i.e., during
    % the "for day=..." looping routine).  These starting values are values taken by
    % the variables at the start, i.e., at the beginning of day=1.
    plantmass = initialPMass; %intial pool size of total live plant mass (g) - user defined
    cumNupt = 0; % total mass (g) of N taken up by plant (N mass in shoot+root) - zero at start
    cumNmort = 0; % total mass (g) of N lost from plant via root mortality - zero at start
    Ncont_sum = 0; % proportional N content of ALL plant tissue (not just newly produced plant tissue).
    %% zero at start (actually undefined at start: Ncont_sum = {cumNupt-cumNmort}/plantmass = 0/0).
    %% One little side note, we assume that the proportional N content of dead organic matter is
    %% the same as Ncont_sum.
    R = initialR; % initial pool size of live root mass (g) - user defined in 'run_Nmodel' script file
    OM = initialOM; %intial mass (g) of organic matter (entirely comprised of dead root matter) - user
    %%  defined in 'run_Nmodel' script file
    rootdeath = mr * R; % day 0 rootdeath based on initialR
    decomp = 0; % rate (g/day) of organic matter lost to decomp  - zero at start
    %% alternatively, think of decomp as a pool (g) whose value changes each day, i.e., an amount
    %% of organic matter that disappears owing to decomposition occurring on any particular day.
    
    clear ecotrack; %'ecotrack' tracks changes in variables through time for a given Nc.
    % It needs to be cleared to restart the time series when Nc changes to a new value.  An entire
    % 'ecotrack' matrix that has d rows (where d = # of days) is collapsed to a single row in the
    % 'simseries' matrix, which is the output argument from this function file.  'ecotrack' is only
    % a local bookkeeping matrix within this function file.
    g=0; %counter for number of days since denitrification began
    
    for day = 1:numdays; % Specify the number of days for which to run the model.
        
        ecotrack(day,1) = day; % Create 'ecotrack.' This matrix keeps track of the value of all
        % state variables over the course of one model run (i.e., from day=1 to day='day') for
        % the given value of Nc.
        
        % *** Determine total N input for day j ******************************************
        % calculate recycled N on day j and add it to hydrologic N (constant for all days)
        Nrec = decomp * Ncont_sum; % recycled N (g/day) = decomp rate of OM * N content of OM
        Nin = Nrec + Nhyd; %N input to system equals sum of hydrologic and recycling inputs of N
        % ********************************************************************************
        ecotrack(day,2) = Nhyd;
        ecotrack(day,3) = Nrec;
        ecotrack(day,4) = Nin;
        
        
        % *** calculate N lost to denitrification ************************
        Ndnt = 0.762 * log(R) - 4.06114; % N lost to denitrification (g/day)
        % empirical relation based on field measurements calculated to g/day
        % assume parameters are reasonable since they are from an empirical relationship
        if Ndnt < 0; % from eqn above for Ndnt, Ndnt is negative for small R (R < 206 g)
            Ndnt = 0; %prevent Ndnt from being negative
        end
        ecotrack(day,5) = Ndnt;
        % ****************************************************************
        if Ndnt > 0; %i.e., once denitrification kicks in (or, once R > 206 such that Ndnt > 0)
            g=g+1; %counts number of days with denitrification occurring
            if g==1;P_noDNT = P;end %sets P_noDNT at value of P on last day of no denitrification
        end
        
        % calculate plant-available N as portion of Nin remaining after denitrification...
        Nav = Nin - Ndnt; % plant-available N = portion of Nin remaining after denitrification
        ecotrack(day,6) = Nav;
        
        % ...then calculate N-dependent variables, which are:
        % (i) plant productivity, (ii) foliar %N, and (iii) prop of prod comprising root matter ***
        P = Pmax * Nav / (kp + Nav); % whole-plant productivity (g/day)
        plantmass = plantmass + P - rootdeath; % update mass of whole plant (shoots plus roots)
        ecotrack(day,7) = P;
        ecotrack(day,8) = plantmass;
        
        if modeltype == 0; % CONSTANT MODEL
            Ncont_new = 0.02; % proportional N composition of newly produced plant tissue
            propR = 0.44; % proportion of newly produced tissue allocated to roots
        else % RESPONSE MODEL
            Ncont_new = Ncontmax * Nav / (kNcont + Nav);
            propR = 0.08 + (c * exp(k1 * Nav));
        end
        ecotrack(day,9) = Ncont_new;
        ecotrack(day,10) = propR;
        % ***************************************************************************************
        
        Nupt = P * Ncont_new; % N uptake by plant (g/day)
        cumNupt = cumNupt + Nupt; % update total mass (g) of N taken up by plants
        Nplant = cumNupt-cumNmort; % mass (g) of N in whole plant
        Ncont_sum = Nplant/plantmass; % recalculate proportional N content of all
        % plant tissue (old and new, roots and shoots)
        ecotrack(day,11) = Nupt;
        ecotrack(day,12) = cumNupt;
        ecotrack(day,13) = Nplant;
        ecotrack(day,14) = Ncont_sum;
        
        Nexp = Nin - Ndnt - Nupt; % amount (g/day) of N exported from gravel bar
        %eqn equivalent to Nex = Nin - Ndnt - Nupt - dNfr/dt, with dNfr/dt = 0 by definition
        retention = Nin - Nexp; % retention of N (g/day)
        reten_prct = 100 * retention / Nin; % percentage retention of N
        ecotrack(day,15) = Nexp;
        ecotrack(day,16) = retention;
        ecotrack(day,17) = reten_prct;
        
        % ******* determine R - pool size of living + dead root biomass ********************
        rootprod = P * propR; % root production (g/day); some proportion (propR) of whole plant production
        rootdeath = mr * R; % loss (death) of root mass (g/day); some proportion (mr) of root mass
        dRdt = rootprod - rootdeath;% dRdt = dR/dt; daily change (g/d) in mass of R
        R = R + dRdt; % update R
        ecotrack(day,18) = rootprod;
        ecotrack(day,19) = rootdeath; %because OMprod=rootdeath, this archives both OMprod & rootdeath
        ecotrack(day,20) = dRdt;
        ecotrack(day,21) = R;
        
        Nmort = rootdeath*Ncont_sum; % mass (g/d) of N lost from plant via root mortality
        cumNmort = cumNmort + Nmort; % update total mass (g) of N lost from plants via root mortality
        ecotrack(day,22) = Nmort;
        ecotrack(day,23) = cumNmort;
        
        OMprod = rootdeath; % production (g/day) of organic matter = loss of roots
        decomp = OM * kd; % decomposition rate (g/day) of organic matter
        dOMdt = OMprod - decomp;% dOMdt = dOM/dt; daily change in mass of OM
        OM = OM + dOMdt; % uptdate OM
        ecotrack(day,24) = decomp;
        ecotrack(day,25) = dOMdt;
        ecotrack(day,26) = OM;
        % **********************************************************************************
        
        if day > 1;
            dPdt = P - ecotrack(day-1,7); %change in plant productivity rate (g/day^2)
            %% NOT change in plant size (g/day)
        else dPdt=0;    
        end
        ecotrack(day,27) = dPdt;
        
        %%% *******************************************************************************
        %%% *******************************************************************************
        if ss_mode==1; % if running model in "steady state" mode, check value of dPdt
            %% If running the "prespecified # of days" mode, skip the following "3-ifs" loop
            % **** following nested "3-ifs" loop stops day-by-day simulation at steady state**
            if Ndnt > 0; %ensures that denitrification has started
                if P_noDNT - P > 0.0001; %ensures that program doesn't prematurely "find" steady state
                    %% immediately after 1st day that denitrification occurs (this premature "finding"
                    %% of steady state might happen after 1st day of denit. because day 1 denit is 
                    %% often so small that dP/dt (g/d^2) is near zero at some high level of P (g/d)
                    if abs(dPdt) < 0.0001; break; end; % breaks the "for days=1:numdays" loop
                end
            end
            % terminates program at steady state, defined as minimal change in the plant production
            % rate (i.e., < 0.0001 g/day^2)  NOTE: steady state is NOT definted as minmal change in
            % plant size (i.e., < 0.0001 g/day).  P can be flat at two points in time: (i) right
            % away before denitrification (DNT) kicks in, and (ii) at the end of the time series
            % after DNT and plant N have stopped changing.  This breaks the for-loop once P is flat
            % at the end of the time series
            % ********************************************************************************
        end
        %%% *******************************************************************************
        %%% *******************************************************************************
        
    end
    
    if abs(dPdt) > 0.0001;disp('WARNING: for [N] ='),disp(Nc),disp('steady state not reached by program time limit');end
    
    days_pre_denit = find(ecotrack(:,5)==0); %gets list of the days before denitrification kicks in
    day1_denit = length(days_pre_denit) + 1; %returns first day that denitrification happens
    daySTDYST = day; %returns first day dPdt (change in rate of plant growth; not change in size of
    % plant) < 0.1 mg / day^2; def'n of steady state
    
    day,Nc
    % 'simseries' archives the value of state variables after 'numdays' days
    % where 'numdays' is user defined
    simseries(n,1) = Nc; %initial N concentration
    simseries(n,2) = sum(ecotrack(:,2)); % cum N (g) input via hydro source thru last day of sim
    simseries(n,3) = sum(ecotrack(:,3)); % cumulative (cum) recycled N (g) thru last day of simulation
    simseries(n,4) = sum(ecotrack(:,4)); % cum N (g) input thru last day of simulation (sim)
    simseries(n,5) = sum(ecotrack(:,5)); % cum N (g) denitrified thru last day of sim
    simseries(n,6) = sum(ecotrack(:,6)); % cum N (g) that had been available for P thru last day of sim
    simseries(n,7) = sum(ecotrack(:,7)); % cum plant production (g) thru last day of sim
    simseries(n,8) = ecotrack(day,8); % plant size (g) at end of last day of sim
    simseries(n,9) = ecotrack(day,9); % proportional N content of plant tissue produced on last day of sim
    simseries(n,10) = ecotrack(day,10); % prop of tissue produced on last day of sim allocated to roots
    simseries(n,11) = sum(ecotrack(:,11)); % cum plant uptake of N (g) thru last day of sim
    simseries(n,12) = ecotrack(day,12); % cum plant uptake of N (g) on last day of sim
    simseries(n,13) = ecotrack(day,13); % mass (g) of N in whole plant on last day of sim
    simseries(n,14) = ecotrack(day,14); % proportional N content of whole plant on last day of sim
    simseries(n,15) = sum(ecotrack(:,15)); % cum N (g) export thru last day of sim
    simseries(n,16) = sum(ecotrack(:,16)); % cum N (g) retained thru last day of sim
    cumNin = Nin * day; % cum Ninput at end of 5 years
    perc_totreten = 100 * sum(ecotrack(:,16)) / cumNin; %percent retention of cum Ninput
    simseries(n,17) = perc_totreten; %percent of cum N input during period of sim that is retained
    simseries(n,18) = sum(ecotrack(:,18)); % cum root production (g) thru last day of sim
    simseries(n,19) = sum(ecotrack(:,19)); % cum root death (and OM production) thru last day of sim
    simseries(n,20) = ecotrack(day,20); % daily rate of change in root mass (dR/dt) on last day of sim
    simseries(n,21) = ecotrack(day,21); %live root mass (g) after last day of sim
    simseries(n,22) = sum(ecotrack(:,22)); % cum N lost (g) from plant via root mortality thru last day of sim
    simseries(n,23) = ecotrack(day,23); % cum N lost (g) from plant via root mortality on last day of sim
    simseries(n,24) = sum(ecotrack(:,24)); % cum OM decomposed (g) thru last day of sim
    simseries(n,25) = ecotrack(day,25); % daily rate of change in OM mass (dOM/dt) on last day of sim
    simseries(n,26) = ecotrack(day,26); % OM mass (g) after last day of sim
    simseries(n,27) = ecotrack(day,27); % rate of change in production (g/day^2) on last day of sim
    simseries(n,28) = day1_denit; % first day that denitrification occurred (i.e., that Ndnt > 0)
    simseries(n,29) = daySTDYST; % day that steady state was reached (first day > day1_denit on which
    %% dP/dt < 0.0001). This variable ("daySTDYST") is only meaningful when model is run in "steady
    %% state" mode. When model is run for a prespecified number of days, daySTDYST will always equal
    %% that prespecified number, regardless of the value of dP/dt.
    simseries(n,30) = simseries(n,5)./simseries(n,16); % proportion of retention via denitrification
    simseries(n,31) = simseries(n,11)./simseries(n,16); % proportion of retention via plant uptake
    simseries(n,32) = simseries(n,5)./simseries(n,11); % denitrification-uptake ratio
    simseries(n,33) = simseries(n,3)./simseries(n,4); % proportion of input from recycling
    
end

% in function file mode, comment out graphing commands
% in script file mode, uncomment graphing commands

% set(0,'DefaultLineLineWidth',2.5)
% figure(4)
% subplot(2,2,1)
% plot (simseries(:,1),simseries(:,4),'k-',simseries(:,1),simseries(:,3),'k:')
% xlabel('N concentration (mg / L)')
% ylabel('N input (g) during 5 y')
% title('solid: recyc+hydro, dot: recycled')
% 
% subplot(2,2,2)
% plot (simseries(:,1),simseries(:,5),'k-',simseries(:,1),simseries(:,11),'k:')
% xlabel('N concentration (mg / L)')
% ylabel('N loss (g) during 5 y')
% title('solid: denitrif, dot: plant uptake')
% 
% subplot(2,2,3)
% plot (simseries(:,1),simseries(:,15),'k-',simseries(:,1),simseries(:,16),'k:')
% xlabel('N concentration (mg / L)')
% ylabel('flux of N (g) during 5y')
% title('solid: N export, dot: N retention')
% 
% subplot(2,2,4)
% plot (simseries(:,1),simseries(:,17),'k-')
% xlabel('N concentration (mg / L)')
% ylabel('% of 5-y input of N retained')
% 
% figure(5)
% subplot(2,2,1)
% plotyy (simseries(:,1),simseries(:,7),simseries(:,1),simseries(:,27))
% [AX,H1,H2] = plotyy(simseries(:,1),simseries(:,7),simseries(:,1),simseries(:,27));
% set(get(AX(1),'Ylabel'),'String','plant production (g) during 5y')
% set(get(AX(2),'Ylabel'),'String','dP/dt (g/d^2) on day 1825')
% set(AX(1),'YColor','black'); set(AX(2),'YColor','black')
% set(H1,'LineStyle','-'); set(H2,'LineStyle',':')
% set(H1,'Color','k'); set(H2,'Color','k')
% xlabel('N concentration (mg / L)')
% title('solid: P, dot: dP/dt')
% 
% subplot(2,2,2)
% plot (simseries(:,1),simseries(:,8),'k-')
% xlabel('N concentration (mg / L)')
% ylabel('plant mass (g) on day 1825')
% 
% subplot(2,2,3)
% plot (simseries(:,1),simseries(:,21),'k-',simseries(:,1),simseries(:,26),'k:')
% xlabel('N concentration (mg / L)')
% ylabel('mass (g) on day 1825')
% title('solid: roots, dot: OM')
% 
% subplot(2,2,4)
% plot (simseries(:,1),simseries(:,20),'k-',simseries(:,1),simseries(:,25),'k:')
% xlabel('N concentration (mg / L)')
% ylabel('change in pool size (g/d) on day 1825')
% title('solid: roots, dot: OM')
% 
% figure(6)
% subplot(2,2,1)
% plot (simseries(:,1),simseries(:,18),'k-',simseries(:,1),simseries(:,19),'k--',simseries(:,1),simseries(:,24),'k:')
% xlabel('N concentration (mg / L)')
% ylabel('flux (g) during 5y')
% title('solid: root prod, dash: root-to-OM, dot: OM decomp')
% 
% subplot(2,2,2)
% plot (simseries(:,1),simseries(:,14),'k-',simseries(:,1),simseries(:,9),'k:')
% xlabel('N concentration (mg / L)')
% ylabel('proportional N content of tissue')
% title('solid: whole plant, dot: produced on day 1825')
% 
% subplot(2,2,3)
% plot (simseries(:,1),simseries(:,10),'k-')
% xlabel('N concentration (mg / L)')
% ylabel('prop day 1825 prod allocated to roots')
% 
% subplot(2,2,4)
% plot (simseries(:,1),simseries(:,12),'k-')
% xlabel('N concentration (mg / L)')
% ylabel('mass of N in plant tissue (g) on day 1825')