%% File contains model equations from "The response of a riparian shrub to
%% changes in N availability:  implications for N retention."  Schade et al.

% Set up to run sensitivity analyses of several uncertain parameters
% at several levels of N

clear
format short
echo off
tic

% ******** determine Nin - rate of hydrologic input of N to gravel bar *************
flowvel = 1; % velocity of flow through gravel bar (m/h)
plant_xsec = 0.158; % eventual cross-sectional area of zone influenced by BIG plant (m2)
% NOTE: to evaluate retention, we hold constant the volume of space being considered, thus
% plant_xsec is set to the 'space to be filled' by the growing plant
Q = flowvel * plant_xsec * 24; % hydro discharge through gravel bar (m3/d)
list_Nc=[.1 .2 .3 .4 .5 .6 .7 .8 .9 1];
list_sensprop=[.25 .5 .75 .9 1 1.1 1.25 1.5 1.75 2 3 4];
% **********************************************************************************

% ******* set parameters for P - whole plant productivity rate *********************
% ******* (actually determine P below, after removing N loss from DNT) *************
Pmax = 10; %max productvity (g/day)
kp = 0.42; %half-sat const of productivity - Ninput relationship
% **********************************************************************************

% ******* determine percN - %N composition of plant foliage ************************
percNmax = 0.04; %max plant percent N
kpercN = 0.25; %half-sat const of plant % N - Ninput relationship
% **********************************************************************************

% ******* determine R_S - root:shoot ratio of plant ********************************
c = 0.69; %intercept parameter of R_S - Ninput relationship; original value 0.69
base_k1 = -1.74 %decay parameter of R_S - Ninput relationship; original value -2.74
% **********************************************************************************

rootdecomp = 0.001;

%load sensit_B.txt;
%rowct = size(sensit_B,1); %counter for rows of output
%sensit = sensit_B;
rowct = 0;

for CR_toggle = 1:2; %toggles between "C"onstant & "R"esponse models
    for SI=1:length(list_sensprop);
        sensprop = list_sensprop(SI);
        k1 = sensprop * base_k1
        
        for n=1:length(list_Nc);
            rowct = rowct + 1; %increment counter for rows of output
            Nc = list_Nc(n) % nitrogen concentration (g/m3 = mg/L) input to gravel bar
            Nin = Q * Nc; % hydrologic input of N to gravel bar (g/day)
            R = 5; % initial pool size of root (living and dead) mass (g)
            shootmass_p = 4 * R; % initial mass (g) of plant - p method: shootmass calculated from plant prod
            clear ecotrack
            g=0; %counter for number of days since denitrification began
            
            for day = 1:9125; %model has 25 years to find steady state
                DNT = 0.762 * log(R) - 4.06114; % denitrification (g/day)
                % empirical relation based on field measurements calculated to g/day
                % assume parameters are reasonable since they are from an empirical relationship
                % (NOTE to self: what is R^2 of this relationship)
                if DNT < 0;
                    DNT = 0; %prevent DNT from being negative
                end
                if DNT > 0;
                    g=g+1; %counts number of days with denitrification
                    if g==1;P_noDNT = P;end %sets P_noDNT at value of P on last day of no denitrification
                end
                
                
                % ****calculate plant available N as portion of Nin remaining after denitrification *****
                % ****then calculate N-dependent variables:  plant product., foliar %N, and root:shoot **
                plantN = Nin - DNT; % plant-available N = portion of Nin remaining after denitrification
                P = Pmax * plantN / (kp + plantN); % whole-plant productivity (g/day)
                shootmass_p = shootmass_p + P;
                % NOTE: plant productivity is a function of N, but not of plant size
                if CR_toggle == 1; %constant model mode
                    percN = 0.02; % percent N value in constant model
                    R_S = 0.44; % root:shoot ratio in constant model
                else %response model mode
                    percN = percNmax * plantN / (kpercN + plantN); % percent N composition of foliar tissue; response model
                    R_S = 0.1 + (c * exp(k1 * plantN)); % root:shoot ratio in reponse model
                end
                % ***************************************************************************************
                
                ecotrack(day,1) = day;
                ecotrack(day,2) = R; %archive root biomass
                ecotrack(day,3) = P; %archive plant productivity rate
                if day > 1;
                    dPdt = P - ecotrack(day-1,3); %change in plant productivity rate; NOT change in plant size
                end
                ecotrack(day,4) = DNT; %archive denitrification rate
                
                Nupt = P * percN; % N uptake by plant (g/day)
                % NOTE: Nupt not a function of plant size
                
                Nex = Nin - DNT - Nupt; % amount of N exported from gravel bar
                %eqn equivalent to Nex = Nin - DNT - Nupt - dN/dt, with dN/dt = 0 by definition
                retention = Nin - Nex; % retention of N (g/day)
                reten_prct = 100 * retention / Nin; % percent retention of N
                
                ecotrack(day,5) = Nex; %archive daily N export
                ecotrack(day,6) = retention; %archive daily N retention
                ecotrack(day,7) = reten_prct; %archive daily percent N retention
                
                % ******* determine R - pool size of living + dead root biomass ********************
                rootprod = P * R_S; % root production (g/day)
                %rootdecomp = 0.001; % per capita root decomposition rate (1/day) - range from 0.0001 - 0.001; incl. decomp of roots and detritus
                dRdt = rootprod - (rootdecomp * R); % dRdt = dR/dt; rate of change of R
                ecotrack(day,8) = dRdt; %archive daily root growth
                R = R + dRdt; % update R
                shootmass_rs = R / R_S;
                % **********************************************************************************
                ecotrack(day,9) = plantN; %archive plant available N throughput (g / day)
                ecotrack(day,10) = Nupt; %archive plant uptake of N (g/day)
                ecotrack(day,11) = R_S; %archive root:shoot ratio
                ecotrack(day,12) = shootmass_p; %archive shootmass_p
                ecotrack(day,13) = shootmass_rs; %archive shootmass_rs
                
                % *********** uncomment following five-line if-loop to make program stop at steady state***
                if DNT > 0; %ensures that denitrification has started
                    if P_noDNT - P > 0.0001; %ensures that program doesn't prematurely "find" steady state immediately after 1st DNT
                        if abs(dPdt) < 0.0001; break; end;
                    end
                end
                % terminates program at steady state, defined as minimal change in the plant production rate
                % (i.e., < 0.0001 g/day^2)  NOTE: steady state is NOT definted as minmal change in plant size
                % (i.e., < 0.0001 g/day).  P can be flat at two points in time: (i) right away before
                % denitrification (DNT) kicks in, and (ii) at the end of the time series after DNT and plant N
                % have stopped changing.  This breaks the for-loop once P is flat at the end of the time series
                % ******************************************************************************************
                
            end
            
            SSflag=0; % SSflag=0 means steady state was reached
            if abs(dPdt) > 0.0001;Nc,disp('WARNING: steady state not reached by program time limit');SSflag=1;end
            
            days_pre_DNT = find(ecotrack(:,4)==0); %returns a list of the days before denitrification kicks in
            dayDNT = length(days_pre_DNT) + 1 %returns first day that denitrification happens
            daySTDYST = day %returns first day dPdt (change in rate of plant growth; not change in size of plant)
            % < 0.1 mg / day^2; def'n of steady state
            
            %'sensit' is a matrix of output from a loop that varies 7 parameters - they are listed in the first
            % seven columns.  The other columns archive final and cumulative values of various state variables
            sensit(rowct,1) = Pmax;
            sensit(rowct,2) = kp;
            sensit(rowct,3) = percNmax;
            sensit(rowct,4) = kpercN;
            sensit(rowct,5) = c;
            sensit(rowct,6) = k1;
            sensit(rowct,7) = rootdecomp;
            sensit(rowct,8) = Nc; %initial N concentration
            sensit(rowct,9) = daySTDYST; %day of steady state
            sensit(rowct,10) = ecotrack(daySTDYST,2); %root biomass (g) when steady state reached
            sensit(rowct,11) = ecotrack(daySTDYST,11); %root:shoot ratio of new production when steady state reached
            sensit(rowct,12) = ecotrack(daySTDYST,12); %plant size (g) at steady state
            sensit(rowct,13) = dayDNT; %day that denitrification kicks in
            sensit(rowct,14) = (ecotrack(daySTDYST,3)); % rate (g/day) of plant production when steady state reached
            sensit(rowct,15) = (ecotrack(daySTDYST,10)); % rate (g/day) of plant uptake of N when steady state reached
            sensit(rowct,16) = (ecotrack(daySTDYST,4)); % denitrification rate (g/day) when steady state reached
            sensit(rowct,17) = (ecotrack(daySTDYST,6)); % N retention rate (g/day) when steady state reached
            sensit(rowct,18) = (ecotrack(daySTDYST,5)); % N export rate (g/day) when steady state reached
            sensit(rowct,19) = (ecotrack(daySTDYST,4))/(ecotrack(daySTDYST,10)); %ratio daily DNT: daily plant N uptake
            sensit(rowct,20) = 100*(ecotrack(daySTDYST,6))/ Nin; % percent daily N input retained at steady state
            sensit(rowct,21) = 100*(ecotrack(daySTDYST,4))/(ecotrack(daySTDYST,6)); % percent daily N retention as denitrification at steady state
            sensit(rowct,22) = 100*(ecotrack(daySTDYST,10))/(ecotrack(daySTDYST,6)); % percent daily N retention as plant N uptake at steady state        
            sensit(rowct,23) = dPdt; % rate of change of rate of plant production (g/day^2) when steady state reached;
            % dPdt is used to define steady state, so this values SHOULD be =< 0.0005
            sensit(rowct,24) = sum(ecotrack(1:daySTDYST,3)); % cumulative plant production when steady state reached
            sensit(rowct,25) = sum(ecotrack(1:daySTDYST,10)); % cumulative plant uptake of N when steady state reached
            sensit(rowct,26) = sum(ecotrack(1:daySTDYST,4)); % cumulative denitrification when steady state reached
            sensit(rowct,27) = sum(ecotrack(1:daySTDYST,6)); % cumulative N retention when steady state reached
            sensit(rowct,28) = sum(ecotrack(1:daySTDYST,5)); % cumulative N export when steady state reached
            cumNin = Nin * daySTDYST; % total Ninput at point when steady state is reached
            perc_totreten = 100 * sum(ecotrack(1:daySTDYST,6)) / cumNin; %percent retention of total N
            sensit(rowct,29) = perc_totreten;
            sensit(rowct,30) = SSflag; %if 0, steady state found; if 1, steady state not reached
        end
    end
end
toc
 % 1st half of rows in sensit from constant model
 % 2nd half of rows in sensit from response model
moderows = length(list_Nc)*length(list_sensprop); %number of rows in each model mode (constant & response)
sensit_C = sensit(1:moderows,:); %constant model output
sensit_R = sensit(moderows+1:2*moderows,:); %response model output
rows = length(list_Nc);

% **** create matrix to compare response and constant models ************************
% **** comparison technique: % that response model deviates from constant model *****
% **** calculate as 100 * (R - C)/ C ************************************************
CR_comp(:,1) = sensit_C(:,1); % Pmax
CR_comp(:,2) = sensit_C(:,2); % kp
CR_comp(:,3) = sensit_C(:,3); % percNmax
CR_comp(:,4) = sensit_C(:,4); % kpercN
CR_comp(:,5) = sensit_C(:,5); % c
CR_comp(:,6) = sensit_C(:,6); % k1
CR_comp(:,7) = sensit_C(:,7); % rootdecomp
CR_comp(:,8) = sensit_C(:,8); % Nc, initial N concentration
CR_comp(:,9) = 100*((sensit_R(:,9)-sensit_C(:,9))./sensit_C(:,9)); % day of steady state; % difference between constant and response models
CR_comp(:,10) = 100*((sensit_R(:,10)-sensit_C(:,10))./sensit_C(:,10)); % root biomass (g) when steady state reached; % diff
CR_comp(:,11) = 100*((sensit_R(:,11)-sensit_C(:,11))./sensit_C(:,11)); % root:shoot ratio of new production when steady state reached; % diff
CR_comp(:,12) = 100*((sensit_R(:,12)-sensit_C(:,12))./sensit_C(:,12)); % plant size (g) at steady state; % diff
CR_comp(:,13) = 100*((sensit_R(:,13)-sensit_C(:,13))./sensit_C(:,13)); % day that denitrification kicks in; % diff
CR_comp(:,14) = 100*((sensit_R(:,14)-sensit_C(:,14))./sensit_C(:,14)); % rate (g/day) of plant production when steady state reached; % diff
CR_comp(:,15) = 100*((sensit_R(:,15)-sensit_C(:,15))./sensit_C(:,15)); % rate (g/day) of plant uptake of N when steady state reached; % diff
CR_comp(:,16) = 100*((sensit_R(:,16)-sensit_C(:,16))./sensit_C(:,16)); % denitrification rate (g/day) when steady state reached; % diff
CR_comp(:,17) = 100*((sensit_R(:,17)-sensit_C(:,17))./sensit_C(:,17)); % N retention rate (g/day) when steady state reached; % diff
CR_comp(:,18) = 100*((sensit_R(:,18)-sensit_C(:,18))./sensit_C(:,18)); % N export rate (g/day) when steady state reached; % diff
CR_comp(:,19) = 100*((sensit_R(:,19)-sensit_C(:,19))./sensit_C(:,19)); % ratio daily DNT: daily plant N uptake; % diff
CR_comp(:,20) = 100*((sensit_R(:,20)-sensit_C(:,20))./sensit_C(:,20)); % percent daily N input retained at steady state; % diff
CR_comp(:,21) = 100*((sensit_R(:,21)-sensit_C(:,21))./sensit_C(:,21)); % percent daily N retention as denitrification at steady state; % diff
CR_comp(:,22) = 100*((sensit_R(:,22)-sensit_C(:,22))./sensit_C(:,22)); % percent daily N retention as plant N uptake at steady state; % diff
CR_comp(:,23) = 100*((sensit_R(:,23)-sensit_C(:,23))./sensit_C(:,23)); % rate of change of rate of plant production (g/day^2) when steady state reached; % diff
CR_comp(:,24) = 100*((sensit_R(:,24)-sensit_C(:,24))./sensit_C(:,24)); % cumulative plant production when steady state reached; % diff
CR_comp(:,25) = 100*((sensit_R(:,25)-sensit_C(:,25))./sensit_C(:,25)); % cumulative plant uptake of N when steady state reached; % diff
CR_comp(:,26) = 100*((sensit_R(:,26)-sensit_C(:,26))./sensit_C(:,26)); % cumulative denitrification when steady state reached; % diff
CR_comp(:,27) = 100*((sensit_R(:,27)-sensit_C(:,27))./sensit_C(:,27)); % cumulative N retention when steady state reached; % diff
CR_comp(:,28) = 100*((sensit_R(:,28)-sensit_C(:,28))./sensit_C(:,28)); % cumulative N export when steady state reached; % diff
CR_comp(:,29) = 100*((sensit_R(:,29)-sensit_C(:,29))./sensit_C(:,29)); % percent cumulative N input retained; % diff
CR_comp(:,30) = sensit_R(:,30) + sensit_C(:,30); % if 0, steady state found; if > 0, steady state not reached for one of the model modes

JDS = 24;

% save sensit_range_k1.txt sensit -ascii -tabs
% save CR_comp_k1.txt CR_comp -ascii -tabs

set(0,'DefaultLineLineWidth',2.5)
figure(1)
subplot(2,2,1)
plot (sensit_C((0*rows+1):(1*rows),8),sensit_C((0*rows+1):(1*rows),JDS),'s-k',sensit_R((0*rows+1):(1*rows),8),sensit_R((0*rows+1):(1*rows),JDS),'o:k')
xlim([0 1])
xlabel('N concentration')
ylabel('days to steady state')
title('k1 = 25% baseline; solid constant; dotted response')

subplot(2,2,2)
plot (sensit_C((3*rows+1):(4*rows),8),sensit_C((3*rows+1):(4*rows),JDS),'s-k',sensit_R((3*rows+1):(4*rows),8),sensit_R((3*rows+1):(4*rows),JDS),'o:k')
xlim([0 1])
xlabel('N concentration')
ylabel('days to steady state')
title('k1 = 90% baseline; solid constant; dotted response')

subplot(2,2,3)
plot (sensit_C((4*rows+1):(5*rows),8),sensit_C((4*rows+1):(5*rows),JDS),'s-k',sensit_R((4*rows+1):(5*rows),8),sensit_R((4*rows+1):(5*rows),JDS),'o:k')
xlim([0 1])
xlabel('N concentration')
ylabel('days to steady state')
title('k1 = baseline (-1.74); solid constant; dotted response')

subplot(2,2,4)
plot (sensit_C((9*rows+1):(10*rows),8),sensit_C((9*rows+1):(10*rows),JDS),'s-k',sensit_R((9*rows+1):(10*rows),8),sensit_R((9*rows+1):(10*rows),JDS),'o:k')
xlim([0 1])
xlabel('N concentration')
ylabel('days to steady state')
title('k1 = double baseline; solid constant; dotted response')

figure(2)
subplot(2,2,1)
plot (CR_comp((0*rows+1):(1*rows),8),CR_comp((0*rows+1):(1*rows),JDS),'s-k')
xlim([0 1])
xlabel('N concentration')
ylabel('days to steady state: % deviation response from constant')
title('k1 = 25%; solid constant; dotted response')

subplot(2,2,2)
plot (CR_comp((3*rows+1):(4*rows),8),CR_comp((3*rows+1):(4*rows),JDS),'s-k')
xlim([0 1])
xlabel('N concentration')
ylabel('days to steady state: % deviation response from constant')
title('k1 = 90%; solid constant; dotted response')

subplot(2,2,3)
plot (CR_comp((4*rows+1):(5*rows),8),CR_comp((4*rows+1):(5*rows),JDS),'s-k')
xlim([0 1])
xlabel('N concentration')
ylabel('days to steady state: % deviation response from constant')
title('k1 = baseline; solid constant; dotted response')

subplot(2,2,4)
plot (CR_comp((9*rows+1):(10*rows),8),CR_comp((9*rows+1):(10*rows),JDS),'s-k')
xlim([0 1])
xlabel('N concentration')
ylabel('days to steady state: % deviation response from constant')
title('k1 = double baseline; solid constant; dotted response')